Package: GSCA
Type: Package
Title: GSCA: Gene Set Context Analysis
Version: 2.42.0
Date: 2015-12-8
Author: Zhicheng Ji, Hongkai Ji
Maintainer: Zhicheng Ji <zji4@jhu.edu>
Description: GSCA takes as input several lists of activated and
        repressed genes. GSCA then searches through a compendium of
        publicly available gene expression profiles for biological
        contexts that are enriched with a specified pattern of gene
        expression. GSCA provides both traditional R functions and
        interactive, user-friendly user interface.
License: GPL(>=2)
LazyLoad: yes
Imports: graphics
Depends: shiny, sp, gplots, ggplot2, reshape2, RColorBrewer, rhdf5,
        R(>= 2.10.0)
Suggests: Affyhgu133aExpr, Affymoe4302Expr, Affyhgu133A2Expr,
        Affyhgu133Plus2Expr
biocViews: GeneExpression, Visualization, GUI
git_url: https://git.bioconductor.org/packages/GSCA
git_branch: RELEASE_3_23
git_last_commit: 98d6c01
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:28:26 UTC; biocbuild
