export(GSCAplot)
export(GSCA)
export(GSCAeda)
export(annotatePeaks)
export(tabSearch)
export(ConstructTG)
export(GSCAui)
importFrom(graphics, abline, legend, plot, points)
import(shiny)
import(sp)
import(gplots)
import(ggplot2)
import(reshape2)
import(RColorBrewer)
import(rhdf5)
